'From OLPC2.0 of ''24 October 2006'' [latest update: #1471] on 15 August 2007 at 10:34:30 am'!
"Change Set:		ip6-mir
Date:			26 June 2007
Author:			Michael Rueger, Ian Piumarta

Image support for IPv6, plus backwards-compatibilty code"!

Object subclass: #NetNameResolver
	instanceVariableNames: ''
	classVariableNames: 'DefaultHostName HaveNetwork ResolverBusy ResolverError ResolverMutex ResolverReady ResolverSemaphore ResolverUninitialized UseOldNetwork '
	poolDictionaries: ''
	category: 'Network-Kernel'!
Object subclass: #OldSocket
	instanceVariableNames: 'semaphore socketHandle readSemaphore writeSemaphore primitiveOnlySupportsOneSemaphore '
	classVariableNames: 'Connected DeadServer InvalidSocket OtherEndClosed Registry RegistryThreshold TCPSocketType ThisEndClosed UDPSocketType Unconnected WaitingForConnection '
	poolDictionaries: ''
	category: 'Network-Kernel'!
ByteArray variableByteSubclass: #SocketAddress
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Network-Kernel'!

!SocketAddress commentStamp: '<historical>' prior: 0!
I represent a socket (network) address consisting of a host internet address and a port number.  My contents are opaque and cannot be interpreted directly.  See the accessing protocol for methods that retrieve the information I contain.!

Object subclass: #SocketAddressInformation
	instanceVariableNames: 'socketAddress addressFamily socketType protocol'
	classVariableNames: 'AddressFamilyINET4 AddressFamilyINET6 AddressFamilyLocal AddressFamilyUnspecified NumericFlag PassiveFlag ProtocolTCP ProtocolUDP ProtocolUnspecified SocketTypeDGram SocketTypeStream SocketTypeUnspecified'
	poolDictionaries: ''
	category: 'Network-Kernel'!

!SocketAddressInformation commentStamp: '<historical>' prior: 0!
I represent a local or remote network service.

Instance Variables
	addressFamily:	<SmallInteger> the address family (unix, inet4, inet6, ...) in which the service address is available.
	protocol:		<SmallInteger> the protocol (tcp, udp, ...) that the service uses.
	socketAddress:	<SocketAddress> the socket address at which the service can be contacted or created.
	socketType:		<SmallInteger> the type (stream, dgram) of the socket that should be created for communication with the service.
!


!AudioChatGUI methodsFor: 'initialization' stamp: 'dgd 2/14/2003 19:48'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	transmitWhileRecording _ false.
	handsFreeTalking _ false.
	mycodec _ GSMCodec new.
	myrecorder _ ChatNotes new.
	mytargetip _ ''.
	
	self start2.
	self changeTalkButtonLabel! !

!AudioChatGUI methodsFor: 'stuff' stamp: 'mir 6/17/2007 20:03'!
connect

	| address |
	mytargetip := FillInTheBlank 
		request: 'Connect to?' 
		initialAnswer: (mytargetip ifNil: ['']).
	mytargetip := (address := NetNameResolver addressForName: mytargetip)
		ifNil: ['']
		ifNotNil: [address hostNumber]! !


!ByteArray methodsFor: 'converting' stamp: 'mir 6/17/2007 23:12'!
asSocketAddress
	^SocketAddress fromOldByteAddress: self! !


!ConnectionQueue methodsFor: 'private' stamp: 'bf 6/29/2007 17:58'!
listenLoop
	"Private!! This loop is run in a separate process. It will establish up to maxQueueLength connections on the given port."
	"Details: When out of sockets or queue is full, retry more frequently, since a socket may become available, space may open in the queue, or a previously queued connection may be aborted by the client, making it available for a fresh connection."
	"Note: If the machine is disconnected from the network while the server is running, the currently waiting socket will go from 'isWaitingForConnection' to 'unconnected', and attempts to create new sockets will fail. When this happens, delete the broken socket and keep trying to create a socket in case the network connection is re-established. Connecting and disconnecting was tested under PPP on Mac system 8.1. It is not if this will work on other platforms."


	| newConnection adressInfos |
	NetNameResolver useOldNetwork
		ifTrue: [^self oldListenLoop].

	adressInfos := SocketAddressInformation
						forHost: '' service: portNumber asString
						flags: SocketAddressInformation passiveFlag
						addressFamily: SocketAddressInformation addressFamilyINET4
						socketType: SocketAddressInformation socketTypeStream
						protocol: SocketAddressInformation protocolTCP.
	"We'll accept four simultanous connections at the same time"
	socket := adressInfos first listenWithBacklog: 4.
	"If the listener is not valid then the we cannot use the
	BSD style accept() mechanism."
	socket isValid ifFalse: [^self oldStyleListenLoop].
	[true] whileTrue: [
		socket isValid ifFalse: [
			"socket has stopped listening for some reason"
			socket destroy.
			(Delay forMilliseconds: 10) wait.
			^self listenLoop ].
		newConnection _ socket waitForAcceptFor: 10.
		(newConnection notNil and:[newConnection isConnected]) ifTrue:
			[accessSema critical: [connections addLast: newConnection].
			newConnection _ nil].
		self pruneStaleConnections]. ! !

!ConnectionQueue methodsFor: 'private' stamp: 'bf 6/29/2007 17:29'!
oldListenLoop
	"Private!! This loop is run in a separate process. It will establish up to maxQueueLength connections on the given port."
	"Details: When out of sockets or queue is full, retry more frequently, since a socket may become available, space may open in the queue, or a previously queued connection may be aborted by the client, making it available for a fresh connection."
	"Note: If the machine is disconnected from the network while the server is running, the currently waiting socket will go from 'isWaitingForConnection' to 'unconnected', and attempts to create new sockets will fail. When this happens, delete the broken socket and keep trying to create a socket in case the network connection is re-established. Connecting and disconnecting was tested under PPP on Mac system 8.1. It is not if this will work on other platforms."


	| newConnection |
	socket _ Socket newTCP.
	"We'll accept four simultanous connections at the same time"
	socket listenOn: portNumber backlogSize: 4.
	"If the listener is not valid then the we cannot use the
	BSD style accept() mechanism."
	socket isValid ifFalse: [^self oldStyleListenLoop].
	[true] whileTrue: [
		socket isValid ifFalse: [
			"socket has stopped listening for some reason"
			socket destroy.
			(Delay forMilliseconds: 10) wait.
			^self oldListenLoop ].
		newConnection _ socket waitForAcceptFor: 10.
		(newConnection notNil and:[newConnection isConnected]) ifTrue:
			[accessSema critical: [connections addLast: newConnection].
			newConnection _ nil].
		self pruneStaleConnections]. ! !


!EToyListenerMorph class methodsFor: 'as yet unclassified' stamp: 'mir 6/17/2007 20:05'!
commResultDeferred: anArrayOfAssociations

	| m ipAddress aDictionary |

	"to be run as part of the UI process in case user interaction is required"

	aDictionary _ Dictionary new.
	anArrayOfAssociations do: [ :each | aDictionary add: each].
	
	aDictionary at: #commFlash ifPresent: [ :ignore | ^self].
	m _ aDictionary at: #message ifAbsent: [^self].
	m = 'OK' ifFalse: [^self].
	ipAddress _ aDictionary at: #ipAddress.

	EToyIncomingMessage new 
		incomingMessgage: (ReadStream on: (aDictionary at: #data)) 
		fromIPAddress: ipAddress

	! !


!EToyPeerToPeer methodsFor: 'sending' stamp: 'mir 6/17/2007 20:04'!
doConnectForSend

	| addr |

	addr _ NetNameResolver addressForName: ipAddress.
	addr ifNil: [
		communicatorMorph commResult: {#message -> ('could not find ',ipAddress)}.
		^false
	].
	socket connectNonBlockingTo: addr port: self class eToyCommunicationsPort.
	[socket waitForConnectionFor: 15]
		on: ConnectionTimedOut
		do: [:ex |
			communicatorMorph commResult: {#message -> ('no connection to ',ipAddress,' (',
				ipAddress,')')}.
			^false].
	^true

! !

!EToyPeerToPeer methodsFor: 'receiving' stamp: 'mir 6/17/2007 20:05'!
receiveDataOn: aSocket for: aCommunicatorMorph

	socket _ aSocket.
	remoteSocketAddress _ socket remoteSocketAddress hostNumber.
	communicatorMorph _ aCommunicatorMorph.
	process _ [
		leftOverData _ ''.
		[self doReceiveData] whileTrue.
		socket closeAndDestroy.
	] newProcess.
	process priority: Processor highIOPriority.
	process resume.
! !


!NebraskaClient methodsFor: 'as yet unclassified' stamp: 'mir 6/17/2007 20:06'!
currentStatusString

	(connection isNil or: [connection isConnected not]) ifTrue: [^'nada'].
	^connection remoteSocketAddress hostNumber,
		' - ',
		(self backlog // 1024) printString,'k'! !

!NebraskaClient methodsFor: 'initialization' stamp: 'mir 6/17/2007 20:06'!
initialize: aConnection

	| remoteAddress userPicture |

	connection := aConnection.
	hand := RemoteControlledHandMorph on: (MorphicEventDecoder on: aConnection).
	hand nebraskaClient: self.
	remoteAddress _ connection remoteSocketAddress hostNumber.
	userPicture _ EToySenderMorph pictureForIPAddress: remoteAddress.
	hand
		userInitials: ((EToySenderMorph nameForIPAddress: remoteAddress) ifNil: ['???'])
		andPicture: (userPicture ifNotNil: [userPicture scaledToSize: 16@20]).
	encoder := CanvasEncoder on: aConnection.
	canvas := RemoteCanvas
		connection: encoder
		clipRect: NebraskaServer extremelyBigRectangle
		transform: MorphicTransform identity! !


!NetNameResolver class methodsFor: 'lookup' stamp: 'mir 6/17/2007 19:27'!
addressesForName: hostName
	"NetNameResolver addressesForName: 'impara.de' "
	| adresses |
	adresses := SocketAddressInformation
		forHost: hostName
		service: ''
		flags: 0
		addressFamily: 0
		socketType: SocketAddressInformation socketTypeStream
		protocol: SocketAddressInformation protocolTCP.
	^adresses! !

!NetNameResolver class methodsFor: 'lookup' stamp: 'mir 6/26/2007 18:29'!
addressForName: hostName
	"NetNameResolver addressForName: 'impara.de' "
	"NetNameResolver addressForName: 'localhost' "
	"NetNameResolver addressForName: '127.0.0.1' "
	| addresses |
	self useOldNetwork
		ifTrue: [^self oldAddressForName: hostName].
	addresses := self addressesForName: hostName.
	^addresses
		ifEmpty: [nil]
		ifNotEmpty: [addresses first socketAddress]! !

!NetNameResolver class methodsFor: 'lookup' stamp: 'mir 6/17/2007 19:52'!
localAddressString
	"Return a string representing the local host address as four decimal bytes delimited with decimal points."
	"NetNameResolver localAddressString"

	^self localHostAddress hostNumber! !

!NetNameResolver class methodsFor: 'lookup' stamp: 'bf 7/19/2007 13:15'!
localHostAddress
	"Return the local address of this host."
	"NetNameResolver localHostAddress"

	^NetNameResolver addressForName: self localHostName! !

!NetNameResolver class methodsFor: 'lookup' stamp: 'mir 6/17/2007 19:32'!
localHostName
	"Return the local name of this host."
	"NetNameResolver localHostName"

	| host |
	host := String new: NetNameResolver primHostNameSize.
	NetNameResolver primHostNameResult: host.
	^host! !

!NetNameResolver class methodsFor: 'lookups-old' stamp: 'mir 6/17/2007 23:14'!
addressForName: hostName timeout: secs
	"Look up the given host name and return its address. Return nil if the address is not found in the given number of seconds."
	"NetNameResolver addressForName: 'create.ucsb.edu' timeout: 30"
	"NetNameResolver addressForName: '100000jobs.de' timeout: 30"
	"NetNameResolver addressForName: '1.7.6.4' timeout: 30"
	"NetNameResolver addressForName: '' timeout: 30 (This seems to return nil?)"

	| deadline result |
	self initializeNetwork.
	self useOldNetwork
		ifFalse: [^self addressForName: hostName].
	"check if this is a valid numeric host address (e.g. 1.2.3.4)"
	result _ self addressFromString: hostName.
	result isNil ifFalse: [^result asSocketAddress].

	"Look up a host name, including ones that start with a digit (e.g. 100000jobs.de or squeak.org)"
	deadline _ Time millisecondClockValue + (secs * 1000).
	"Protect the execution of this block, as the ResolverSemaphore is used for both parts of the transaction."
	self resolverMutex
		critical: [
			(self waitForResolverReadyUntil: deadline)
				ifTrue: [
					self primStartLookupOfName: hostName.
					(self waitForCompletionUntil: deadline)
						ifTrue: [result _ self primNameLookupResult]
						ifFalse: [(NameLookupFailure hostName: hostName) signal: 'Could not resolve the server named: ', hostName]]
				ifFalse: [(NameLookupFailure hostName: hostName) signal: 'Could not resolve the server named: ', hostName]].
	^result asSocketAddress! !

!NetNameResolver class methodsFor: 'lookups-old' stamp: 'mir 6/26/2007 18:30'!
addressFromString: addressString
	"Return the internet address represented by the given string. The string should contain four positive decimal integers delimited by periods, commas, or spaces, where each integer represents one address byte. Return nil if the string is not a host address in an acceptable format."
	"NetNameResolver addressFromString: '1.2.3.4'"
	"NetNameResolver addressFromString: '1,2,3,4'"
	"NetNameResolver addressFromString: '1 2 3 4'"

	| newAddr s byte delimiter |
	newAddr _ ByteArray new: 4.
	s _ ReadStream on: addressString.
	s skipSeparators.
	1 to: 4 do: [:i |
		byte _ self readDecimalByteFrom: s.
		byte = nil ifTrue: [^ nil].
		newAddr at: i put: byte.
		i < 4 ifTrue: [
			delimiter _ s next.
			((delimiter = $.) or: [(delimiter = $,) or: [delimiter = $ ]])
				ifFalse: [^ nil]]].
	^ newAddr
! !

!NetNameResolver class methodsFor: 'lookups-old' stamp: 'mir 6/26/2007 18:53'!
oldAddressForName: aString
	"NetNameResolver oldAddressForName: 'vpri.org' "
	^self addressForName: aString timeout: 60! !

!NetNameResolver class methodsFor: 'lookups-old' stamp: 'mir 6/26/2007 18:30'!
stringFromAddress: addr
	"Return a string representing the given host address as four decimal bytes delimited with decimal points."
	"NetNameResolver stringFromAddress: NetNameResolver localHostAddress"

	| s |
	s _ WriteStream on: ''.
	1 to: 3 do: [ :i | (addr at: i) printOn: s. s nextPut: $.].
	(addr at: 4) printOn: s.
	^ s contents
! !

!NetNameResolver class methodsFor: 'network initialization' stamp: 'mir 6/26/2007 18:26'!
initializeNetwork
	"Initialize the network drivers and record the semaphore to be used by the resolver. Do nothing if the network is already initialized. Evaluate the given block if network initialization fails."
	"NetNameResolver initializeNetwork" 

	| semaIndex |
	self resolverStatus = ResolverUninitialized
		ifFalse: [^HaveNetwork _ true].  "network is already initialized"

	HaveNetwork _ false.	"in case abort"
	ResolverSemaphore _ Semaphore new.
	semaIndex _ Smalltalk registerExternalObject: ResolverSemaphore.

	"result is nil if network initialization failed, self if it succeeds"
	(self primInitializeNetwork: semaIndex)
		ifNil: [NoNetworkError signal: 'failed network initialization']
		ifNotNil: [HaveNetwork _ true].

	UseOldNetwork := [NetNameResolver primHostNameSize. false]
		on: Error
		do: [:ex | ex return: true]
! !

!NetNameResolver class methodsFor: 'tests' stamp: 'mir 1/5/2007 18:06'!
testIPv6
	"NetNameResolver testIPv6"
	| infos addr sock size host serverSocket listeningSocket clientSocket |
	World findATranscript: World currentEvent.
	Transcript clear.
	"Transcript show: SmalltalkImage current listLoadedModules; cr."
	self initializeNetwork.
	Transcript show: '---- host name ----'; cr.
	size := NetNameResolver primHostNameSize.
	host := String new: size.
	NetNameResolver primHostNameResult: host.
	Transcript show: host; cr.
	Transcript show: '---- address information ----'; cr.
	Transcript show: (infos := SocketAddressInformation
						forHost: 'localhost' service: 'echo' flags: 0
						addressFamily: 0 socketType: 0 protocol: 0) printString; cr.
	Transcript show: '---- port manipulation ----'; cr.
	addr := infos first socketAddress.
	Transcript show: addr port printString; cr.
	addr port: 1234.
	Transcript show: addr port printString; cr.
	Transcript show: addr printString; cr.
	Transcript show: '---- client socket ----'; cr.
	Transcript show: (infos := SocketAddressInformation
						forHost: 'localhost' service: 'echo' flags: 0
						addressFamily: 0
						socketType: SocketAddressInformation socketTypeStream
						protocol: SocketAddressInformation protocolTCP) printString; cr.
	infos do: [:info |
		Transcript show: 'Trying ', info printString, '... '.
		(sock := info connect) notNil
			ifTrue:
				[sock sendData: 'hello' count: 5.
				 Transcript show: sock receiveData printString.
				 sock close; destroy].
		Transcript cr].
	Transcript show: '---- localhost defaults: loopback and wildcard addresses ----'; cr.
	Transcript show: (SocketAddress loopbacks) printString; cr.
	Transcript show: (SocketAddress wildcards) printString; cr.
	Transcript show: (SocketAddress loopback4) printString; cr.
	Transcript show: (SocketAddress wildcard4) printString; cr.
	Transcript show: '---- impossible constraints ----'; cr.
	Transcript show: (SocketAddressInformation
						forHost: 'localhost' service: 'echo' flags: 0
						addressFamily:	0
						socketType:		SocketAddressInformation socketTypeDGram
						protocol:		SocketAddressInformation protocolTCP) printString; cr.
	Transcript show: '---- INET4 client-server ----'; cr.
	Transcript show: (infos := SocketAddressInformation
						forHost: '' service: '4242'
						flags:			SocketAddressInformation passiveFlag
						addressFamily:	SocketAddressInformation addressFamilyINET4
						socketType:		SocketAddressInformation socketTypeStream
						protocol:		SocketAddressInformation protocolTCP) printString; cr.
	listeningSocket := infos first listenWithBacklog: 5.
	Transcript show: (infos := SocketAddressInformation
						forHost: 'localhost' service: '4242'
						flags:			0
						addressFamily:	SocketAddressInformation addressFamilyINET4
						socketType:		SocketAddressInformation socketTypeStream
						protocol:		SocketAddressInformation protocolTCP) printString; cr.
	clientSocket := infos first connect.
	serverSocket := listeningSocket accept.
	serverSocket sendData: 'Hi there!!' count: 9.
	Transcript show: clientSocket receiveData; cr.
	Transcript nextPutAll: 'client side local/remote: ';
		print: clientSocket localSocketAddress; space;
		print: clientSocket remoteSocketAddress; cr.
	Transcript nextPutAll: 'server side local/remote: ';
		print: serverSocket localSocketAddress; space;
		print: serverSocket remoteSocketAddress; cr;
		endEntry.
	clientSocket close; destroy.
	serverSocket close; destroy.
	listeningSocket close; destroy.
	Transcript show: '---- INET6 client-server ----'; cr.
	Transcript show: (infos := SocketAddressInformation
						forHost: '' service: '4242'
						flags:			SocketAddressInformation passiveFlag
						addressFamily:	SocketAddressInformation addressFamilyINET6
						socketType:		SocketAddressInformation socketTypeStream
						protocol:		SocketAddressInformation protocolTCP) printString; cr.
	infos isEmpty
		ifTrue: [Transcript show: 'FAIL -- CANNOT CREATE INET6 SERVER'; cr]
		ifFalse:
			[listeningSocket := infos first listenWithBacklog: 5.
			Transcript show: (infos := SocketAddressInformation
								forHost: 'localhost' service: '4242'
								flags:			0
								addressFamily:	SocketAddressInformation addressFamilyINET6
								socketType:		SocketAddressInformation socketTypeStream
								protocol:		SocketAddressInformation protocolTCP) printString; cr.
			clientSocket := infos first connect.
			serverSocket := listeningSocket accept.
			serverSocket sendData: 'Hi there!!' count: 9.
			Transcript show: clientSocket receiveData; cr.
			Transcript nextPutAll: 'client side local/remote: ';
				print: clientSocket localSocketAddress; space;
				print: clientSocket remoteSocketAddress; cr.
			Transcript nextPutAll: 'server side local/remote: ';
				print: serverSocket localSocketAddress; space;
				print: serverSocket remoteSocketAddress; cr;
				endEntry.
			clientSocket close; destroy.
			serverSocket close; destroy.
			listeningSocket close; destroy].
	Transcript show: '---- trivial tests done ---'; cr.! !

!NetNameResolver class methodsFor: 'tests' stamp: 'mir 1/5/2007 18:18'!
testPort80
	"NetNameResolver testPort80"
	| infos |
	Transcript show: (infos := SocketAddressInformation
						forHost: 'localhost' service: '80' flags: 0
						addressFamily: 0 socketType: 0 protocol: 0) printString; cr.
	Transcript show: (infos := SocketAddressInformation
						forHost: '::1' service: '80' flags: 0
						addressFamily: 0 socketType: 0 protocol: 0) printString; cr.
! !

!NetNameResolver class methodsFor: 'private' stamp: 'ikp 6/8/2007 18:12'!
nextSocketAddressInformation

	| addrSize addr info |
	addrSize := self primGetAddressInfoSize.
	addrSize < 0 ifTrue: [^nil].
	addr := SocketAddress new: addrSize.
	self primGetAddressInfoResult: addr.
	info := SocketAddressInformation
		withSocketAddress: addr
		family: self primGetAddressInfoFamily
		type: self primGetAddressInfoType
		protocol: self primGetAddressInfoProtocol.
	self primGetAddressInfoNext.
	^info! !

!NetNameResolver class methodsFor: 'private' stamp: 'bf 6/27/2007 11:38'!
useOldNetwork
	^UseOldNetwork ~~ false! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:00'!
primGetAddressInfoFamily

	<primitive: 'primitiveResolverGetAddressInfoFamily' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 17:24'!
primGetAddressInfoHost: hostName service: servName flags: flags family: family type: type protocol: protocol

	<primitive: 'primitiveResolverGetAddressInfo' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:19'!
primGetAddressInfoNext

	<primitive: 'primitiveResolverGetAddressInfoNext' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:00'!
primGetAddressInfoProtocol

	<primitive: 'primitiveResolverGetAddressInfoProtocol' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 17:41'!
primGetAddressInfoResult: socketAddress

	<primitive: 'primitiveResolverGetAddressInfoResult' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 17:30'!
primGetAddressInfoSize

	<primitive: 'primitiveResolverGetAddressInfoSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:00'!
primGetAddressInfoType

	<primitive: 'primitiveResolverGetAddressInfoType' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:34'!
primGetNameInfoHostResult: aString

	<primitive: 'primitiveResolverGetNameInfoHostResult' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:34'!
primGetNameInfoHostSize

	<primitive: 'primitiveResolverGetNameInfoHostSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:35'!
primGetNameInfoServiceResult: aString

	<primitive: 'primitiveResolverGetNameInfoServiceResult' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 18:34'!
primGetNameInfoServiceSize

	<primitive: 'primitiveResolverGetNameInfoServiceSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:36'!
primGetNameInfo: socketAddress flags: flags

	<primitive: 'primitiveResolverGetNameInfo' module: 'SocketPlugin'>
	flags == 0 ifTrue: [^self primGetNameInfo: socketAddress
						flags: SocketAddressInformation numericFlag].
	self primitiveFailed! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 09:05'!
primHostNameResult: aString

	<primitive: 'primitiveResolverHostNameResult' module: 'SocketPlugin'>
	self primitiveFailed
! !

!NetNameResolver class methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 09:04'!
primHostNameSize

	<primitive: 'primitiveResolverHostNameSize' module: 'SocketPlugin'>
	self primitiveFailed
! !


!NetworkTerminalMorph methodsFor: 'layout' stamp: 'mir 6/17/2007 20:07'!
acceptDroppingMorph: morphToDrop event: evt

	| myCopy outData null |

	(morphToDrop isKindOf: NewHandleMorph) ifTrue: [			"don't send these"
		^morphToDrop rejectDropMorphEvent: evt.
	].
	self eToyRejectDropMorph: morphToDrop event: evt.		"we don't really want it"

	"7 mar 2001 - remove #veryDeepCopy"
	myCopy _ morphToDrop.	"gradient fills require doing this second"
	myCopy setProperty: #positionInOriginatingWorld toValue: morphToDrop position.

	outData _ myCopy eToyStreamedRepresentationNotifying: nil.
	null _ String with: 0 asCharacter.
	EToyPeerToPeer new 
		sendSomeData: {
			EToyIncomingMessage typeMorph,null. 
			Preferences defaultAuthorName,null.
			outData
		}
		to: connection remoteSocketAddress hostNumber
		for: self.
! !


!OldSocket methodsFor: 'accessing' stamp: 'mir 6/17/2007 19:56'!
peerName
	"Return the name of the host I'm connected to, or nil if its name isn't known to the domain name server or the request times out."
	"Note: Slow. Calls the domain name server, taking up to 20 seconds to time out. Even when sucessful, delays of up to 13 seconds have been observed during periods of high network load." 

	^self remoteSocketAddress hostName! !

!OldSocket methodsFor: 'connection open/close' stamp: 'mir 6/26/2007 18:55'!
connectTo: hostAddress port: port
	"Initiate a connection to the given port at the given host address. This operation will return immediately; follow it with waitForConnectionUntil: to wait until the connection is established."

	NetNameResolver useOldNetwork
		ifTrue: [	| status |
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [self error: 'Socket status must Unconnected before opening a new connection'].

	self primSocket: socketHandle connectTo: hostAddress port: port.
]
		ifFalse: [
			hostAddress port: port.
			self connectTo: hostAddress]! !

!OldSocket methodsFor: 'waiting' stamp: 'mir 6/17/2007 21:12'!
waitForConnectionFor: timeout ifTimedOut: timeoutBlock
	"Wait up until the given deadline for a connection to be established. Return true if it is established by the deadline, false if not."

	| status deadline |
	deadline := Socket deadlineSecs: timeout.
	status _ self primSocketConnectionStatus: socketHandle.
	[(status = WaitingForConnection) and: [Time millisecondClockValue < deadline]]
		whileTrue: [
			semaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue).
			status _ self primSocketConnectionStatus: socketHandle].

	status = Connected ifFalse: [^timeoutBlock value]
! !

!OldSocket methodsFor: 'other' stamp: 'mir 6/17/2007 20:21'!
initializeNetwork
	self class initializeNetwork! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 09:35'!
bindTo: aSocketAddress

	| status |
	self initializeNetwork.
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [InvalidSocketStatusException signal: 'Socket status must Unconnected when binding it to an address'].

	self primSocket: socketHandle bindTo: aSocketAddress.
! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/8/2007 22:11'!
connectNonBlockingTo: aSocketAddress

	| status |
	self initializeNetwork.
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [InvalidSocketStatusException signal: 'Socket status must Unconnected before opening a new connection'].

	self primSocket: socketHandle connectTo: aSocketAddress.
! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/8/2007 22:09'!
connectTo: aSocketAddress

	self connectTo: aSocketAddress waitForConnectionFor: Socket standardTimeout! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/8/2007 22:10'!
connectTo: aSocketAddress waitForConnectionFor: timeout 

	self connectNonBlockingTo: aSocketAddress.
	self
		waitForConnectionFor: timeout
		ifTimedOut: [ConnectionTimedOut signal: 'Cannot connect to ', aSocketAddress printString]! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 09:39'!
listenWithBacklog: backlogSize

	| status |
	self initializeNetwork.
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [InvalidSocketStatusException signal: 'Socket status must Unconnected before it can listen for connections'].

	self primSocket: socketHandle listenWithBacklog: backlogSize.
! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 10:28'!
localSocketAddress

	| size addr |
	size := self primSocketLocalAddressSize: socketHandle.
	addr := SocketAddress new: size.
	self primSocket: socketHandle localAddressResult: addr.
	^addr! !

!OldSocket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 10:28'!
remoteSocketAddress

	| size addr |
	size := self primSocketRemoteAddressSize: socketHandle.
	addr := SocketAddress new: size.
	self primSocket: socketHandle remoteAddressResult: addr.
	^addr! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:24'!
primSocketLocalAddressSize: handle

	<primitive: 'primitiveSocketLocalAddressSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:24'!
primSocketRemoteAddressSize: handle

	<primitive: 'primitiveSocketRemoteAddressSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 09:36'!
primSocket: socketID bindTo: socketAddress

	<primitive: 'primitiveSocketBindTo' module: 'SocketPlugin'>
	self primitiveFailed
! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 22:08'!
primSocket: socketID connectTo: socketAddress

	<primitive: 'primitiveSocketConnectTo' module: 'SocketPlugin'>
	self primitiveFailed
! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 09:39'!
primSocket: socketID listenWithBacklog: backlogSize

	<primitive: 'primitiveSocketListenWithBacklog' module: 'SocketPlugin'>
	self primitiveFailed
! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:23'!
primSocket: socketID localAddressResult: socketAddress

	<primitive: 'primitiveSocketLocalAddressResult' module: 'SocketPlugin'>
	self primitiveFailed
! !

!OldSocket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:23'!
primSocket: socketID remoteAddressResult: socketAddress

	<primitive: 'primitiveSocketRemoteAddressResult' module: 'SocketPlugin'>
	self primitiveFailed
! !


!OldSocket class methodsFor: 'class initialization' stamp: 'mir 1/5/2007 18:37'!
initialize
	"Socket initialize"

	"Socket Types"
	TCPSocketType _ 0.
	UDPSocketType _ 1.

	"Socket Status Values"
	InvalidSocket _ -1.
	Unconnected _ 0.
	WaitingForConnection _ 1.
	Connected _ 2.
	OtherEndClosed _ 3.
	ThisEndClosed _ 4.

! !


!RemoteHandMorph methodsFor: 'connections' stamp: 'mir 6/17/2007 20:10'!
startTransmittingEvents
	"Attempt to broadcast events from this hand to a remote hand on the host with the given address. This method just creates the new socket and initiates a connection; it does not wait for the other end to answer."
	(sendSocket notNil and:[sendSocket isConnected]) ifTrue:[^self].
	Transcript
		show: 'Connecting to remote WorldMorph at ';
		show: self remoteHostAddress hostNumber, ' ...'; cr.
	sendSocket _ OldSimpleClientSocket new.
	sendSocket connectTo: self remoteHostAddress port: 54323.
	sendState _ #opening.
	owner primaryHand addEventListener: self.! !

!RemoteHandMorph methodsFor: 'connections' stamp: 'mir 6/17/2007 20:10'!
startTransmittingEventsTo: remoteAddr
	"Attempt to broadcast events from this hand to a remote hand on the host with the given address. This method just creates the new socket and initiates a connection; it does not wait for the other end to answer."
	remoteAddress _ remoteAddr.
	(sendSocket notNil and:[sendSocket isConnected]) ifTrue:[^self].
	Transcript
		show: 'Connecting to remote WorldMorph at ';
		show: self remoteHostAddress hostNumber, ' ...'; cr.
	sendSocket _ OldSimpleClientSocket new.
	sendSocket connectTo: self remoteHostAddress port: 54323.
	sendState _ #opening.
	owner primaryHand addEventListener: self.! !

!RemoteHandMorph methodsFor: 'event handling' stamp: 'mir 6/17/2007 20:07'!
transmitEvent: aMorphicEvent
	"Transmit the given event to all remote connections."
	| firstEvt |
	self readyToTransmit ifFalse: [^ self].
	self lastEventTransmitted = aMorphicEvent ifTrue: [^ self].
	sendBuffer ifNil: [sendBuffer _ WriteStream on: (String new: 10000)].
	sendBuffer nextPutAll: aMorphicEvent storeString; cr.
	self lastEventTransmitted: aMorphicEvent.

	sendSocket isConnected ifTrue:[
		sendState = #opening ifTrue: [
			"connection established; disable TCP delays on sends"
			sendSocket setOption: 'TCP_NODELAY' value: true.
			"send worldExtent as first event"
			firstEvt _ MorphicUnknownEvent type: #worldBounds argument: self worldBounds extent.
			sendSocket sendData: firstEvt storeString, (String with: Character cr).
			Transcript
				show: 'Connection established with remote WorldMorph at ';
				show: sendSocket remoteSocketAddress hostNumber; cr.
			sendState _ #connected].
		sendSocket sendData: sendBuffer contents.
	] ifFalse: [
		owner primaryHand removeEventListener: self.
		sendState = #connected ifTrue: [
			"other end has closed; close our end"
			Transcript
				show: 'Closing connection with remote WorldMorph at ';
				show: sendSocket remoteSocketAddress hostNumber; cr.
			sendSocket close.
		sendState _ #closing]].

	sendBuffer reset.
! !


!Socket methodsFor: 'initialize-destroy' stamp: 'mir 6/26/2007 18:44'!
initialize: socketType family: family
	"Initialize a new socket handle. If socket creation fails, socketHandle will be set to nil."
	| semaIndex readSemaIndex writeSemaIndex |

	NetNameResolver useOldNetwork ifTrue: [^self initialize: socketType].
	primitiveOnlySupportsOneSemaphore _ false.
	semaphore _ Semaphore new.
	readSemaphore _ Semaphore new.
	writeSemaphore _ Semaphore new.
	semaIndex _ Smalltalk registerExternalObject: semaphore.
	readSemaIndex _ Smalltalk registerExternalObject: readSemaphore.
	writeSemaIndex _ Smalltalk registerExternalObject: writeSemaphore.
	socketHandle _
		self primSocketCreateNetwork: family
			type: socketType
			receiveBufferSize: 8000
			sendBufSize: 8000
			semaIndex: semaIndex
			readSemaIndex: readSemaIndex
			writeSemaIndex: writeSemaIndex.

	socketHandle = nil ifTrue: [  "socket creation failed"
		Smalltalk unregisterExternalObject: semaphore.
		Smalltalk unregisterExternalObject: readSemaphore.
		Smalltalk unregisterExternalObject: writeSemaphore.
		readSemaphore _ writeSemaphore _ semaphore _ nil
	] ifFalse:[self register].
! !

!Socket methodsFor: 'accessing' stamp: 'mir 6/17/2007 19:56'!
peerName
	"Return the name of the host I'm connected to, or nil if its name isn't known to the domain name server or the request times out."
	"Note: Slow. Calls the domain name server, taking up to 20 seconds to time out. Even when sucessful, delays of up to 13 seconds have been observed during periods of high network load." 

	^self remoteSocketAddress hostName! !

!Socket methodsFor: 'connection open/close' stamp: 'mir 6/26/2007 18:52'!
connectTo: hostAddress port: port
	"Initiate a connection to the given port at the given host address.
	Waits until the connection is established or time outs."

	NetNameResolver useOldNetwork
		ifTrue: [self connectTo: hostAddress port: port waitForConnectionFor: Socket standardTimeout]
		ifFalse: [
			hostAddress port: port.
			self connectTo: hostAddress]! !

!Socket methodsFor: 'connection open/close' stamp: 'mir 6/17/2007 20:10'!
connectTo: hostAddress port: port waitForConnectionFor: timeout 
	"Initiate a connection to the given port at the given host 
	address. Waits until the connection is established or time outs."
	self connectNonBlockingTo: hostAddress port: port.
	self
		waitForConnectionFor: timeout
		ifTimedOut: [ConnectionTimedOut signal: 'Cannot connect to '
					, self remoteHostAddress hostNumber , ':' , port asString]! !

!Socket methodsFor: 'waiting' stamp: 'mir 6/17/2007 21:18'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. Return true if it is established by the deadline, false if not."

	| status |
	status _ self primSocketConnectionStatus: socketHandle.
	[(status = WaitingForConnection) and: [Time millisecondClockValue < deadline]]
		whileTrue: [
			semaphore waitTimeoutMSecs: (deadline - Time millisecondClockValue).
			status _ self primSocketConnectionStatus: socketHandle].

	^ status = Connected
! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 09:35'!
bindTo: aSocketAddress

	| status |
	self initializeNetwork.
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [InvalidSocketStatusException signal: 'Socket status must Unconnected when binding it to an address'].

	self primSocket: socketHandle bindTo: aSocketAddress.
! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/8/2007 22:11'!
connectNonBlockingTo: aSocketAddress

	| status |
	self initializeNetwork.
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [InvalidSocketStatusException signal: 'Socket status must Unconnected before opening a new connection'].

	self primSocket: socketHandle connectTo: aSocketAddress.
! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/8/2007 22:09'!
connectTo: aSocketAddress

	self connectTo: aSocketAddress waitForConnectionFor: Socket standardTimeout! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/8/2007 22:10'!
connectTo: aSocketAddress waitForConnectionFor: timeout 

	self connectNonBlockingTo: aSocketAddress.
	self
		waitForConnectionFor: timeout
		ifTimedOut: [ConnectionTimedOut signal: 'Cannot connect to ', aSocketAddress printString]! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 09:39'!
listenWithBacklog: backlogSize

	| status |
	self initializeNetwork.
	status _ self primSocketConnectionStatus: socketHandle.
	(status == Unconnected)
		ifFalse: [InvalidSocketStatusException signal: 'Socket status must Unconnected before it can listen for connections'].

	self primSocket: socketHandle listenWithBacklog: backlogSize.
! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 10:28'!
localSocketAddress

	| size addr |
	size := self primSocketLocalAddressSize: socketHandle.
	addr := SocketAddress new: size.
	self primSocket: socketHandle localAddressResult: addr.
	^addr! !

!Socket methodsFor: 'ipv6' stamp: 'ikp 6/9/2007 10:28'!
remoteSocketAddress

	| size addr |
	size := self primSocketRemoteAddressSize: socketHandle.
	addr := SocketAddress new: size.
	self primSocket: socketHandle remoteAddressResult: addr.
	^addr! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:24'!
primSocketLocalAddressSize: handle

	<primitive: 'primitiveSocketLocalAddressSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:24'!
primSocketRemoteAddressSize: handle

	<primitive: 'primitiveSocketRemoteAddressSize' module: 'SocketPlugin'>
	self primitiveFailed
! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 09:36'!
primSocket: socketID bindTo: socketAddress

	<primitive: 'primitiveSocketBindTo' module: 'SocketPlugin'>
	self primitiveFailed
! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/8/2007 22:08'!
primSocket: socketID connectTo: socketAddress

	<primitive: 'primitiveSocketConnectTo' module: 'SocketPlugin'>
	self primitiveFailed
! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 09:39'!
primSocket: socketID listenWithBacklog: backlogSize

	<primitive: 'primitiveSocketListenWithBacklog' module: 'SocketPlugin'>
	self primitiveFailed
! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:23'!
primSocket: socketID localAddressResult: socketAddress

	<primitive: 'primitiveSocketLocalAddressResult' module: 'SocketPlugin'>
	self primitiveFailed
! !

!Socket methodsFor: 'primitives-ipv6' stamp: 'ikp 6/9/2007 10:23'!
primSocket: socketID remoteAddressResult: socketAddress

	<primitive: 'primitiveSocketRemoteAddressResult' module: 'SocketPlugin'>
	self primitiveFailed
! !


!Socket class methodsFor: 'instance creation' stamp: 'mir 1/5/2007 18:27'!
newTCP
	"Create a socket and initialise it for TCP"
	^self newTCP: SocketAddressInformation addressFamilyINET4! !

!Socket class methodsFor: 'instance creation' stamp: 'ikp 6/8/2007 23:02'!
newTCP: family
	"Create a socket and initialise it for TCP"
	self initializeNetwork.
	^[ super new initialize: TCPSocketType family: family ]
		repeatWithGCIf: [ :socket | socket isValid not ]! !

!Socket class methodsFor: 'instance creation' stamp: 'mir 1/5/2007 18:27'!
newUDP
	"Create a socket and initialise it for UDP"
	^self newUDP: SocketAddressInformation addressFamilyINET4! !

!Socket class methodsFor: 'instance creation' stamp: 'mir 6/17/2007 21:20'!
newUDP: family
	"Create a socket and initialise it for UDP"
	self initializeNetwork.
	^[ super new initialize: UDPSocketType family: family ]
		repeatWithGCIf: [ :socket | socket isValid not ]! !

!Socket class methodsFor: 'instance creation' stamp: 'mir 1/5/2007 18:21'!
tcpCreateIfFail: failBlock
	"Attempt to create a new socket. If successful, return the new socket. Otherwise, return the result of evaluating the given block. Socket creation can fail if the network isn't available or if there are not sufficient resources available to create another socket."

	| sock |
	self initializeNetwork.
	sock _ self newTCP.
	sock isValid ifFalse: [^ failBlock value].
	^ sock
! !

!Socket class methodsFor: 'instance creation' stamp: 'mir 1/5/2007 18:22'!
udpCreateIfFail: failBlock
	"Attempt to create a new socket. If successful, return the new socket. Otherwise, return the result of evaluating the given block. Socket creation can fail if the network isn't available or if there are not sufficient resources available to create another socket."

	| sock |
	self initializeNetwork.
	sock _ self newUDP.
	sock isValid ifFalse: [^ failBlock value].
	^ sock
! !

!Socket class methodsFor: 'examples' stamp: 'mir 6/17/2007 21:19'!
timeTest
	"Socket timeTest"

	| serverName serverAddr s |
	Transcript show: 'initializing network ... '.
	self initializeNetwork.
	Transcript
		show: 'ok';
		cr.
	serverName := FillInTheBlank request: 'What is your time server?'
				initialAnswer: 'localhost'.
	serverName isEmpty 
		ifTrue: 
			[^Transcript
				show: 'never mind';
				cr].
	serverAddr := NetNameResolver addressForName: serverName timeout: 10.
	serverAddr = nil 
		ifTrue: [self error: 'Could not find the address for ' , serverName].
	s := self new.
	Transcript
		show: '---------- Connecting ----------';
		cr.
	s connectTo: serverAddr port: 13.	"13 is the 'daytime' port number"
	s waitForConnectionUntil: (self deadlineSecs: 1).
	Transcript show: 'the time server reports: ' , s receiveData.
	s closeAndDestroy.
	Transcript
		show: '---------- Connection Closed ----------';
		cr! !

!Socket class methodsFor: 'examples' stamp: 'mir 6/17/2007 21:19'!
timeTestUDP
	"Socket timeTestUDP"

	| serverName serverAddr s |
	Transcript show: 'initializing network ... '.
	self initializeNetwork.
	Transcript
		show: 'ok';
		cr.
	serverName := FillInTheBlank request: 'What is your time server?'
				initialAnswer: 'localhost'.
	serverName isEmpty 
		ifTrue: 
			[^Transcript
				show: 'never mind';
				cr].
	serverAddr := NetNameResolver addressForName: serverName timeout: 10.
	serverAddr = nil 
		ifTrue: [self error: 'Could not find the address for ' , serverName].
	s := self newUDP.	"a 'random' port number will be allocated by the system"
	"Send a packet to the daytime port and it will reply with the current date."
	Transcript
		show: '---------- Sending datagram from port ' , s port printString 
					, ' ----------';
		cr.
	s 
		sendData: '!!'
		toHost: serverAddr
		port: 13.	"13 is the daytime service"
	Transcript show: 'the time server reports: ' , s receiveData.
	s closeAndDestroy.
	Transcript
		show: '---------- Socket closed ----------';
		cr! !


!SocketAddress methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:41'!
hostName

	| size name |
	NetNameResolver primGetNameInfo: self flags: 0.
	size := NetNameResolver primGetNameInfoHostSize.
	name := String new: size.
	NetNameResolver primGetNameInfoHostResult: name.
	^name! !

!SocketAddress methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:41'!
hostNumber

	| size name |
	NetNameResolver primGetNameInfo: self flags: 1.
	size := NetNameResolver primGetNameInfoHostSize.
	name := String new: size.
	NetNameResolver primGetNameInfoHostResult: name.
	^name! !

!SocketAddress methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:41'!
serviceName

	| size name |
	NetNameResolver primGetNameInfo: self flags: 0.
	size := NetNameResolver primGetNameInfoServiceSize.
	name := String new: size.
	NetNameResolver primGetNameInfoServiceResult: name.
	^name! !

!SocketAddress methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:41'!
serviceNumber

	| size name |
	NetNameResolver primGetNameInfo: self flags: 1.
	size := NetNameResolver primGetNameInfoServiceSize.
	name := String new: size.
	NetNameResolver primGetNameInfoServiceResult: name.
	^name! !

!SocketAddress methodsFor: 'primitives' stamp: 'ikp 6/9/2007 08:21'!
port

	<primitive: 'primitiveSocketAddressGetPort' module: 'SocketPlugin'>
	self primitiveFailed
! !

!SocketAddress methodsFor: 'primitives' stamp: 'ikp 6/9/2007 08:21'!
port: anInteger

	<primitive: 'primitiveSocketAddressSetPort' module: 'SocketPlugin'>
	self primitiveFailed
! !

!SocketAddress methodsFor: 'printing' stamp: 'ikp 6/8/2007 22:43'!
printOn: aStream

	aStream
		nextPutAll: self hostNumber;
		nextPut: $(; nextPutAll: self hostName; nextPut: $);
		nextPut: $,;
		nextPutAll: self serviceNumber;
		nextPut: $(; nextPutAll: self serviceName; nextPut: $)! !

!SocketAddress methodsFor: 'converting' stamp: 'mir 6/17/2007 23:13'!
asSocketAddress
! !


!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:04'!
loopback4

	^self loopbacks4 first! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:04'!
loopback6

	^self loopbacks6 first! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:02'!
loopbacks

	^SocketAddressInformation forHost: '' service: '0'
		flags:			0
		addressFamily:	0
		socketType:		0
		protocol:		0! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/11/2007 10:35'!
loopbacks4

	^SocketAddressInformation forHost: 'localhost' service: ''
		flags:			0
		addressFamily:	SocketAddressInformation addressFamilyINET4
		socketType:		0
		protocol:		0! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:02'!
loopbacks6

	^SocketAddressInformation forHost: '' service: '0'
		flags:			0
		addressFamily:	SocketAddressInformation addressFamilyINET6
		socketType:		0
		protocol:		0! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:04'!
wildcard4

	^self wildcards4 first! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:04'!
wildcard6

	^self wildcards6 first! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:03'!
wildcards

	^SocketAddressInformation forHost: '' service: '0'
		flags:			SocketAddressInformation passiveFlag
		addressFamily:	0
		socketType:		0
		protocol:		0! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/11/2007 10:40'!
wildcards4

	^SocketAddressInformation forHost: '' service: '0'
		flags:			SocketAddressInformation passiveFlag
		addressFamily:	SocketAddressInformation addressFamilyINET4
		socketType:		SocketAddressInformation socketTypeStream
		protocol:		0! !

!SocketAddress class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 10:03'!
wildcards6

	^SocketAddressInformation forHost: '' service: '0'
		flags:			SocketAddressInformation passiveFlag
		addressFamily:	SocketAddressInformation addressFamilyINET6
		socketType:		0
		protocol:		0! !

!SocketAddress class methodsFor: 'instance creation' stamp: 'mir 6/26/2007 18:33'!
fromOldByteAddress: byteArray
	^self newFrom: byteArray! !


!SocketAddressInformation methodsFor: 'initialize-release' stamp: 'ikp 6/8/2007 18:11'!
initSocketAddress: aSocketAddress family: familyInteger type: typeInteger protocol: protocolInteger

	socketAddress := aSocketAddress.
	addressFamily := familyInteger.
	socketType := typeInteger.
	protocol := protocolInteger.! !

!SocketAddressInformation methodsFor: 'circuit setup' stamp: 'ikp 6/8/2007 23:04'!
connect

	| sock |
	socketType == SocketTypeStream ifFalse: [^nil].
	sock := Socket newTCP: addressFamily.
	sock connectTo: socketAddress.
	sock waitForConnectionFor: Socket standardTimeout
		ifTimedOut: [ConnectionTimedOut signal: 'Cannot connect to ', self printString].
	^sock! !

!SocketAddressInformation methodsFor: 'circuit setup' stamp: 'ikp 6/9/2007 09:52'!
listenWithBacklog: backlog

	| sock |
	(socketType == SocketTypeStream and: [protocol == ProtocolTCP]) ifFalse: [self error: 'cannot listen'].
	sock := Socket newTCP: addressFamily.
	sock bindTo: socketAddress.
	sock listenWithBacklog: 5.
	^sock! !

!SocketAddressInformation methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:20'!
addressFamilyName

	^#(unspecified local inet4 inet6) at: addressFamily + 1! !

!SocketAddressInformation methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:20'!
protocolName

	^#(unspecified tcp udp) at: socketType + 1! !

!SocketAddressInformation methodsFor: 'accessing' stamp: 'ikp 6/9/2007 08:35'!
socketAddress

	^socketAddress! !

!SocketAddressInformation methodsFor: 'accessing' stamp: 'ikp 6/8/2007 18:20'!
socketTypeName

	^#(unspecified stream dgram) at: socketType + 1! !

!SocketAddressInformation methodsFor: 'printing' stamp: 'ikp 6/8/2007 21:54'!
printOn: aStream

	aStream
		print: socketAddress;
		nextPut: $-; nextPutAll: self addressFamilyName;
		nextPut: $-; nextPutAll: self socketTypeName;
		nextPut: $-; nextPutAll: self protocolName! !


!SocketAddressInformation class methodsFor: 'instance creation' stamp: 'ikp 6/8/2007 23:07'!
forHost: hostName service: servName flags: flags addressFamily: family socketType: type protocol: protocol

	| result addr |
	NetNameResolver initializeNetwork.
	NetNameResolver primGetAddressInfoHost: hostName service: servName flags: flags family: family type: type protocol: protocol.
	result := OrderedCollection new.
	[(addr := NetNameResolver nextSocketAddressInformation) notNil] whileTrue: [result add: addr].
	^result! !

!SocketAddressInformation class methodsFor: 'instance creation' stamp: 'ikp 6/8/2007 18:11'!
withSocketAddress: socketAddress family: family type: type protocol: protocol

	^self new initSocketAddress: socketAddress family: family type: type protocol: protocol! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
addressFamilyINET4

	^AddressFamilyINET4! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
addressFamilyINET6

	^AddressFamilyINET6! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
addressFamilyLocal

	^AddressFamilyLocal! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:01'!
addressFamilyUnspecified

	^AddressFamilyUnspecified! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 09:13'!
numericFlag

	^NumericFlag! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/9/2007 09:12'!
passiveFlag

	^PassiveFlag! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
protocolTCP

	^ProtocolTCP! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
protocolUDP

	^ProtocolUDP! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
protocolUnspecified

	^ProtocolUnspecified! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
socketTypeDGram

	^SocketTypeDGram! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
socketTypeStream

	^SocketTypeStream! !

!SocketAddressInformation class methodsFor: 'accessing' stamp: 'ikp 6/8/2007 22:02'!
socketTypeUnspecified

	^SocketTypeUnspecified! !

!SocketAddressInformation class methodsFor: 'class initialization' stamp: 'ikp 6/9/2007 09:12'!
initialize			"SocketAddressInformation initialize"

	NumericFlag := 1.
	PassiveFlag := 2.
	AddressFamilyUnspecified := 0.
	AddressFamilyLocal := 1.
	AddressFamilyINET4 := 2.
	AddressFamilyINET6 := 3.
	SocketTypeUnspecified := 0.
	SocketTypeStream := 1.
	SocketTypeDGram := 2.
	ProtocolUnspecified := 0.
	ProtocolTCP := 1.
	ProtocolUDP := 2.! !

SocketAddressInformation initialize!
Socket removeSelector: #remoteAddress!
Socket removeSelector: #waitForAcceptFor:ifTimedOut:!
OldSimpleClientSocket class removeSelector: #finger:!
OldSimpleClientSocket class removeSelector: #forkingRemoteCursorSender!
OldSimpleClientSocket class removeSelector: #httpTestHost:port:url:!
OldSimpleClientSocket class removeSelector: #nntpTest!
OldSimpleClientSocket class removeSelector: #popTest!
OldSimpleClientSocket class removeSelector: #remoteCursorReceiver!
OldSimpleClientSocket class removeSelector: #remoteCursorTest!
OldSimpleClientSocket class removeSelector: #timeTest!
OldSocket class removeSelector: #clientServerTestUDP!
OldSocket class removeSelector: #clientServerTestUDP2!
OldSocket class removeSelector: #ensureNetworkConnected!
OldSocket initialize!
OldSocket class removeSelector: #loopbackTest!
OldSocket class removeSelector: #pingPorts:on:timeOutSecs:!
OldSocket class removeSelector: #ping:!
OldSocket class removeSelector: #registryThreshold!
OldSocket class removeSelector: #registryThreshold:!
OldSocket class removeSelector: #remoteTestClientTCP!
OldSocket class removeSelector: #remoteTestClientTCPOpenClose1000!
OldSocket class removeSelector: #remoteTestClientTCPOpenClosePutGet!
OldSocket class removeSelector: #remoteTestClientUDP!
OldSocket class removeSelector: #remoteTestServerTCP!
OldSocket class removeSelector: #remoteTestServerTCPOpenClose1000!
OldSocket class removeSelector: #remoteTestServerTCPOpenClosePutGet!
OldSocket class removeSelector: #remoteTestServerTCPUsingAccept!
OldSocket class removeSelector: #remoteTestServerUDP!
OldSocket class removeSelector: #remoteTestServerUDP2!
OldSocket class removeSelector: #remoteTestSinkTCP!
OldSocket class removeSelector: #sendTest!
OldSocket class removeSelector: #timeTest!
OldSocket class removeSelector: #timeTestUDP!
OldSocket class removeSelector: #timeTestUDP2!
OldSocket class removeSelector: #timeTestUDP3!

!OldSocket class reorganize!
('class initialization' initialize)
('instance creation' acceptFrom: createIfFail: new newTCP newUDP tcpCreateIfFail: udpCreateIfFail:)
('network initialization' initializeNetwork initializeNetworkIfFail:)
('tests' newAcceptCheck)
('utilities' deadlineSecs: deadServer deadServer: nameForWellKnownTCPPort: pingPortsOn: standardDeadline wildcardAddress wildcardPort)
('registry' register: registry unregister:)
!

OldSocket removeSelector: #remoteAddress!
OldSocket removeSelector: #waitForAcceptUntil:!
Object subclass: #OldSocket
	instanceVariableNames: 'semaphore socketHandle readSemaphore writeSemaphore primitiveOnlySupportsOneSemaphore'
	classVariableNames: 'Connected DeadServer InvalidSocket OtherEndClosed Registry TCPSocketType ThisEndClosed UDPSocketType Unconnected WaitingForConnection'
	poolDictionaries: ''
	category: 'Network-Kernel'!
ObjectSocket removeSelector: #remoteAddress!
NetNameResolver class removeSelector: #nameForAddress:timeout:!

!NetNameResolver class reorganize!
('class initialization' initialize)
('lookup' addressesForName: addressForName: localAddressString localHostAddress localHostName)
('lookups-old' addressForName:timeout: addressFromString: oldAddressForName: promptUserForHostAddress promptUserForHostAddressDefault: resolverError resolverStatus stringFromAddress:)
('network initialization' initializeNetwork primInitializeNetwork:)
('primitives' primAbortLookup primAddressLookupResult primLocalAddress primNameLookupResult primNameResolverError primNameResolverStatus primStartLookupOfAddress: primStartLookupOfName:)
('tests' testIPv6 testPort80)
('private' nextSocketAddressInformation readDecimalByteFrom: resolverMutex useOldNetwork waitForCompletionUntil: waitForResolverReadyUntil:)
('primitives-ipv6' primGetAddressInfoFamily primGetAddressInfoHost:service:flags:family:type:protocol: primGetAddressInfoNext primGetAddressInfoProtocol primGetAddressInfoResult: primGetAddressInfoSize primGetAddressInfoType primGetNameInfoHostResult: primGetNameInfoHostSize primGetNameInfoServiceResult: primGetNameInfoServiceSize primGetNameInfo:flags: primHostNameResult: primHostNameSize)
!

Object subclass: #NetNameResolver
	instanceVariableNames: ''
	classVariableNames: 'DefaultHostName HaveNetwork ResolverBusy ResolverError ResolverMutex ResolverReady ResolverSemaphore ResolverUninitialized UseOldNetwork'
	poolDictionaries: ''
	category: 'Network-Kernel'!
